@@ -856,6 +856,19 @@ int fsys::linkinfo(const char *path, char *buffer, size_t size)
 #endif
 }
 
+int fsys::hardlink(const char *path, const char *target)
+{
+#ifdef _MSWINDOWS_
+    if(!CreateHardLink(target, path, NULL))
+        return remapError();
+    return 0;
+#else
+    if(::link(path, target))
+        return remapError();
+    return 0;
+#endif
+}
+
 int fsys::link(const char *path, const char *target)
 {
 #if defined(_MSWINDOWS_)
@@ -872,10 +885,10 @@ int fsys::link(const char *path, const char *target)
 
     memset(reparse, 0, sizeof(reparse));
     LOCAL_REPARSE_DATA_BUFFER *rb = (LOCAL_REPARSE_DATA_BUFFER*)&reparse;
-    
+
     if(!MultiByteToWideChar(CP_THREAD_ACP, MB_PRECOMPOSED, dest, lstrlenA(dest) + 1, rb->PathBuffer, lstrlenA(dest) + 1))
         return remapError();
-    
+
     len = lstrlenW(rb->PathBuffer) * 2;
     rb->ReparseTag = IO_REPARSE_TAG_MOUNT_POINT;
     rb->ReparseDataLength = len + 12;
@@ -884,10 +897,10 @@ int fsys::link(const char *path, const char *target)
     h = CreateFile(target, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING,
         FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OPEN_REPARSE_POINT, 0);
     if(!h || h == INVALID_HANDLE_VALUE)
-        return remapError();
+        return hardlink(path, target);
     if(!DeviceIoControl(h, FSCTL_SET_REPARSE_POINT, (LPVOID)rb, rb->ReparseDataLength + FIELD_OFFSET(LOCAL_REPARSE_DATA_BUFFER, SubstituteNameOffset), NULL, 0, &size, 0)) {
         CloseHandle(h);
-        return remapError();
+        return hardlink(path, target);
     }
     CloseHandle(h);
     return 0;
@@ -1376,3 +1389,13 @@ void fsys::set(fd_t handle)
     ptr = NULL;
     error = 0;
 }
+
+fd_t fsys::release(void)
+{
+    fd_t save = fd;
+
+    fd = INVALID_HANDLE_VALUE;
+    ptr = NULL;
+    error = 0;
+    return save;
+}
