@@ -86,6 +86,8 @@ static char **_orig = NULL;
 static OrderedIndex _index;
 static const char *_domain = NULL;
 static shell::exitproc_t _exitproc = NULL;
+static shell::numeric_t numeric_mode = shell::NO_NUMERIC;
+static long numeric_value = 0l;
 
 shell::pipeio::pipeio()
 {
@@ -636,6 +638,7 @@ static const char *msgs[] = {
     _TEXT("unknown command option"),
     _TEXT("option already used"),
     _TEXT("invalid argument used"),
+    _TEXT("numeric value already set"),
     NULL};
 
 const char *shell::errmsg(errmsg_t id)
@@ -648,6 +651,17 @@ void shell::errmsg(errmsg_t id, const char *text)
     msgs[id] = shell::text(text);
 }
 
+void shell::setNumeric(numeric_t mode)
+{
+    numeric_mode = mode;
+    numeric_value = 0l;
+}
+
+long shell::getNumeric(void)
+{
+    return numeric_value;
+}
+
 unsigned shell::count(char **argv)
 {
     unsigned count = 0;
@@ -827,13 +841,51 @@ char **shell::getargv(char **argv)
     const char *value;
     const char *err;
     unsigned argp = 0;
+    bool skip;
 
     while(argv[argp]) {
+        skip = false;
         if(eq(argv[argp], "--")) {
             ++argp;
             break;
         }
         arg = opt = argv[argp];
+
+        switch(numeric_mode) {
+        case shell::NUMERIC_DASH:
+        case shell::NUMERIC_ALL:
+            if(opt[0] == '-' && opt[1] >= '0' && opt[1] <= '9') {
+                if(numeric_value)
+                    shell::errexit(1, "*** %s: %s: %s\n",
+                        _argv0, opt, errmsg(shell::NUMERIC_SET));
+                numeric_value = atol(opt);
+                skip = true;
+            }
+            break;
+        default:
+            break;
+        }
+
+        switch(numeric_mode) {
+        case shell::NUMERIC_PLUS:
+        case shell::NUMERIC_ALL:
+            if(opt[0] == '+' && opt[1] >= '0' && opt[1] <= '9') {
+                if(numeric_value)
+                    shell::errexit(1, "*** %s: %s: %s\n",
+                        _argv0, opt, errmsg(shell::NUMERIC_SET));
+                numeric_value = atol(++opt);
+                skip = true;
+            }
+            break;
+        default:
+            break;
+        }
+
+        if(skip) {
+            ++argp;
+            continue;
+        }
+
         if(*arg != '-')
             break;
 
@@ -1218,6 +1270,16 @@ void shell::relocate(const char *argv0)
 {
 }
 
+String shell::userid(void)
+{
+    char buf[128];
+    DWORD size = sizeof(buf);
+
+    String::set(buf, sizeof(buf), "nobody");
+    GetUserName(buf, &size);
+    return str(buf);
+}
+
 String shell::path(path_t id)
 {
     char buf[512];
@@ -1279,17 +1341,17 @@ String shell::path(path_t id)
         break;
     case PROGRAM_TEMP:
         snprintf(buf, sizeof(buf), "$$%ld$$.tmp", (long)GetCurrentProcessId());
-        result = str("c:\\temp") + str(buf);
+        result = str("c:\\temp\\") + str(buf);
         break;
     case SYSTEM_ETC:
         if(GetEnvironmentVariable("SystemRoot", buf, sizeof(buf)))
             result = str(buf) + "\\etc";
         break;
     case SYSTEM_CFG:
-        result ^= UCOMMON_CFGPATH;
+        result = path(SYSTEM_PREFIX, UCOMMON_CFGPATH);
         break;
     case SYSTEM_VAR:
-        result ^= UCOMMON_VARPATH;
+        result = path(SYSTEM_PREFIX, UCOMMON_VARPATH);
         break;
     case SYSTEM_PREFIX:
         result ^= UCOMMON_PREFIX;
@@ -1660,6 +1722,16 @@ void shell::relocate(const char *argv0)
 #endif
 }
 
+String shell::userid(void)
+{
+    const char *id = ::getenv("LOGNAME");
+
+    if(!id)
+        id = "nobody";
+
+    return str(id);
+}
+
 String shell::path(path_t id)
 {
     string_t result = "";
@@ -1671,10 +1743,10 @@ String shell::path(path_t id)
 
     switch(id) {
     case SYSTEM_CERTIFICATES:
-        result = str(SSL_CERTS);
+        result = path(SYSTEM_PREFIX, SSL_CERTS);
         break;
     case SYSTEM_KEYS:
-        result = str(SSL_PRIVATE);
+        result = path(SYSTEM_PREFIX, SSL_PRIVATE);
         break;
     case USER_DEFAULTS:
         home = ::getenv("HOME");
@@ -1689,7 +1761,7 @@ String shell::path(path_t id)
         result = str(home);
         break;
     case SERVICE_DATA:
-        result = str(UCOMMON_VARPATH "/lib/") + _domain;
+        result = path(SYSTEM_PREFIX, UCOMMON_VARPATH "/lib/") + _domain;
         break;
     case USER_DATA:
         home = ::getenv("HOME");
@@ -1723,7 +1795,7 @@ String shell::path(path_t id)
 #endif
         break;
     case SERVICE_CACHE:
-        result = str(UCOMMON_VARPATH "/cache/") + _domain;
+        result = path(SYSTEM_PREFIX, UCOMMON_VARPATH "/cache/") + _domain;
         break;
     case PROGRAM_CONFIG:
         home = ::getenv("HOME");
@@ -1743,21 +1815,21 @@ String shell::path(path_t id)
 #endif
         break;
     case SERVICE_CONFIG:
-        result = str(UCOMMON_CFGPATH) + "/" + _domain + ".conf";
+        result = path(SYSTEM_PREFIX, UCOMMON_CFGPATH "/") + _domain + ".conf";
         break;
     case SYSTEM_TEMP:
         result ^= "/tmp";
         break;
     case PROGRAM_TEMP:
         snprintf(buf, sizeof(buf), ".$$%ld$$.tmp", (long)getpid());
-        result = str("/tmp") + str(buf);
+        result = str("/tmp/") + str(buf);
         break;
     case SYSTEM_ETC:
     case SYSTEM_CFG:
-        result ^= UCOMMON_CFGPATH;
+        result = path(SYSTEM_PREFIX, UCOMMON_CFGPATH);
         break;
     case SYSTEM_VAR:
-        result ^= UCOMMON_VARPATH;
+        result = path(SYSTEM_PREFIX, UCOMMON_VARPATH);
         break;
     case SYSTEM_PREFIX:
         result ^= system_prefix;
@@ -2155,7 +2227,7 @@ void shell::bind(const char *name)
     const char *prior = _domain;
 
     _domain = name;
-    locale = path(SYSTEM_SHARE) + str("/locale");
+    locale = path(SYSTEM_SHARE) + "/locale";
 
     if(!prior) {
         setlocale(LC_ALL, "");
