@@ -258,6 +258,12 @@ public:
      */
     void set(fd_t descriptor);
 
+    /**
+     * Release descriptor, do not close.
+     * @return handle being released.
+     */
+    fd_t release(void);
+
     /**
      * Set the position of a file descriptor.
      * @param offset from start of file or "end" to append.
@@ -497,6 +503,14 @@ public:
      */
     static int link(const char *path, const char *target);
 
+    /**
+     * Create a hard link.
+     * @param path to create link to.
+     * @param target of link.
+     * @return error number or 0 on success.
+     */
+    static int hardlink(const char *path, const char *target);
+
     /**
      * Read a symbolic link to get it's target.
      * @param path of link.
