@@ -92,7 +92,7 @@ public:
     /**
      * Error table index.
      */
-    typedef enum {NOARGS = 0, NOARGUMENT, INVARGUMENT, BADOPTION, OPTION_USED, BAD_VALUE} errmsg_t;
+    typedef enum {NOARGS = 0, NOARGUMENT, INVARGUMENT, BADOPTION, OPTION_USED, BAD_VALUE, NUMERIC_SET} errmsg_t;
 
     /**
      * Type of error logging we are using.
@@ -104,6 +104,11 @@ public:
      */
     typedef enum {FAIL = 0, ERR, WARN, NOTIFY, INFO, DEBUG0} loglevel_t;
 
+    /**
+     * Numeric mode of parser.
+     */
+    typedef enum {NO_NUMERIC, NUMERIC_PLUS, NUMERIC_DASH, NUMERIC_ALL} numeric_t;
+
     /**
      * Path types to retrieve.
      */
@@ -577,6 +582,10 @@ public:
      */
     shell(size_t pagesize = 0);
 
+    static void setNumeric(numeric_t);
+
+    static long getNumeric(void);
+
     /**
      * Display shell options.
      */
@@ -613,6 +622,12 @@ public:
      */
     static String path(path_t id);
 
+    /**
+     * Get the system login id.
+     * @return login id.
+     */
+    static String userid(void);
+
     /**
      * Get a merged path.  If the path requested is a full path, then
      * the prefix is ignored.
